#include "precomp.h"

CSiUtilExUSB::CSiUtilExUSB(HRESULT* phr)
    : m_pTypeInfo(nullptr), m_pszSerialNumber(nullptr), m_lRefCount(1), m_blDisableDialogBoxes(FALSE)
{
    ::LockServer();

    ITypeLib* pTlb = nullptr;
    if(SUCCEEDED(*phr = LoadRegTypeLib(LIBID_SiUtilExLib, 1, 0, LOCALE_USER_DEFAULT, &pTlb)))
    {
        if(SUCCEEDED(*phr = pTlb->GetTypeInfoOfGuid(IID_ISiUtilExUSB, &m_pTypeInfo)))
        {
            if(nullptr == (m_pszSerialNumber = new (std::nothrow) char[1]))
            {
                *phr = E_OUTOFMEMORY;
            }
            else
            {
                m_pszSerialNumber[0] = '\0';
            }
        }
        pTlb->Release();
    }

    // This is due to a bug in SiUtil.dll.
    // (Reported many many times but not yet fixed)
    DWORD dw;
    ::USBDebugDevices(&dw);
}

CSiUtilExUSB::~CSiUtilExUSB()
{
    if(m_pTypeInfo)
        m_pTypeInfo->Release();

    delete [] m_pszSerialNumber;
    ::UnlockServer();
}

IFACEMETHODIMP
CSiUtilExUSB::QueryInterface(REFIID riid, void** ppvObject)
{
    __try
    {
        *ppvObject = nullptr;

        if(IsEqualIID(riid, IID_IUnknown))
            *ppvObject = GetUnknown();
        else if(IsEqualIID(riid, IID_IDispatch))
            *ppvObject = static_cast<IDispatch*>(this);
        else if(IsEqualIID(riid, IID_ISiUtilExUSB))
            *ppvObject = static_cast<ISiUtilExUSB*>(this);
        else if(IsEqualIID(riid, IID_IProvideClassInfo))
            *ppvObject = static_cast<IProvideClassInfo*>(this);
        else if(IsEqualIID(riid, IID_ISupportErrorInfo))
            *ppvObject = static_cast<ISupportErrorInfo*>(this);

        if(*ppvObject)
        {
            static_cast<IUnknown*>(*ppvObject)->AddRef();
            return(S_OK);
        }
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(E_NOINTERFACE);
}

IFACEMETHODIMP_(ULONG)
CSiUtilExUSB::AddRef()
{
    return(InterlockedIncrement(&m_lRefCount));
}

IFACEMETHODIMP_(ULONG)
CSiUtilExUSB::Release()
{
    ULONG ulResult;

    if(0 == (ulResult = InterlockedDecrement(&m_lRefCount)))
        delete this;

    return(ulResult);
}

IFACEMETHODIMP
CSiUtilExUSB::GetTypeInfoCount(UINT* pctinfo)
{
    if(IsBadWritePtrT(pctinfo))
        return(E_POINTER);

    *pctinfo = 1;
    return(S_OK);
}

IFACEMETHODIMP
CSiUtilExUSB::GetTypeInfo(UINT iTInfo, LCID, ITypeInfo** ppTInfo)
{
    if(IsBadWritePtrT(ppTInfo))
        return(E_POINTER);

    if(iTInfo)
    {
        *ppTInfo = nullptr;
        return(DISP_E_BADINDEX);
    }

    *ppTInfo = m_pTypeInfo;
    (*ppTInfo)->AddRef();
    return(S_OK);
}

IFACEMETHODIMP
CSiUtilExUSB::GetIDsOfNames(REFIID, LPOLESTR* rgszNames, UINT cNames, LCID, DISPID* rgDispId)
{
    return(m_pTypeInfo->GetIDsOfNames(rgszNames, cNames, rgDispId));
}

IFACEMETHODIMP
CSiUtilExUSB::Invoke(DISPID dispIdMember, REFIID, LCID lcid, WORD wFlags, DISPPARAMS* pDispParams, VARIANT* pVarResult, EXCEPINFO* pExcepInfo, UINT* puArgErr)
{
    CThreadLocale loc(lcid);
    UINT uArgErr;
    return(m_pTypeInfo->Invoke(static_cast<ISiUtilExUSB*>(this), dispIdMember, wFlags, pDispParams, pVarResult, pExcepInfo, (puArgErr) ? puArgErr : &uArgErr));
}

IFACEMETHODIMP
CSiUtilExUSB::get_SerialNumber(BSTR* sSerialNum)
{
    if(IsBadWritePtrT(sSerialNum))
        return(RaiseCOMException(DISPID_SIUTILEX_SERIALNUMBER, E_POINTER));

    const HRESULT hr = BSTRFromStringA(m_pszSerialNumber, sSerialNum);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_SERIALNUMBER, hr));
}

IFACEMETHODIMP
CSiUtilExUSB::put_SerialNumber(BSTR sSerialNum)
{
    const int nLen = WideCharToMultiByte(CP_ACP, WC_NO_BEST_FIT_CHARS, sSerialNum, -1, nullptr, 0, nullptr, nullptr);

    if(!nLen)
        return(RaiseCOMException(DISPID_SIUTILEX_SERIALNUMBER, HRESULT_FROM_WIN32(GetLastError())));

    char* pszTemp = new (std::nothrow) char[static_cast<unsigned>(nLen)];

    if(!pszTemp)
        return(RaiseCOMException(DISPID_SIUTILEX_SERIALNUMBER, E_OUTOFMEMORY));

    if(WideCharToMultiByte(CP_ACP, WC_NO_BEST_FIT_CHARS, sSerialNum, -1, m_pszSerialNumber, nLen, nullptr, nullptr))
    {
        delete [] m_pszSerialNumber;
        m_pszSerialNumber = pszTemp;
        return(S_OK);
    }

    const HRESULT hr = HRESULT_FROM_WIN32(GetLastError());
    delete [] pszTemp;
    return(RaiseCOMException(DISPID_SIUTILEX_SERIALNUMBER, hr));
}

IFACEMETHODIMP
CSiUtilExUSB::get_DisableDialogBoxes(VARIANT_BOOL* pblDisable)
{
    if(IsBadWritePtrT(pblDisable))
        return(RaiseCOMException(DISPID_SIUTILEX_DISABLEDIALOGBOXES, E_POINTER));

    *pblDisable = (m_blDisableDialogBoxes) ? VARIANT_TRUE : VARIANT_FALSE;
    return(S_OK);
}

IFACEMETHODIMP
CSiUtilExUSB::put_DisableDialogBoxes(VARIANT_BOOL blDisable)
{
    m_blDisableDialogBoxes = (blDisable != VARIANT_FALSE);
    return(S_OK);
}

IFACEMETHODIMP
CSiUtilExUSB::get_Version(BSTR* pbstrVersion)
{
    if(IsBadWritePtrT(pbstrVersion))
        return(RaiseCOMException(DISPID_SIUTILEX_VERSION, E_INVALIDARG));

    *pbstrVersion = nullptr;
    const char* str;
    HRESULT hr = ::GetUSBDLLVersion(&str);

    if(SUCCEEDED(hr))
        hr = BSTRFromStringA(str, pbstrVersion);

    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_VERSION, hr));
}

IFACEMETHODIMP
CSiUtilExUSB::Connect(enum ECprotocol proto, long nPowerTarget)
{
    const HRESULT hr = ::ConnectUSB(m_pszSerialNumber, proto, static_cast<int>(nPowerTarget), m_blDisableDialogBoxes);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_CONNECT, hr));
}

IFACEMETHODIMP
CSiUtilExUSB::Disconnect()
{
    const HRESULT hr = ::DisconnectUSB();
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_DISCONNECT, hr));
}

IFACEMETHODIMP
CSiUtilExUSB::Connected(VARIANT_BOOL* pblConnected)
{
    if(IsBadWritePtrT(pblConnected))
        return(RaiseCOMException(DISPID_SIUTILEX_CONNECTED, E_POINTER));

    *pblConnected = (::Connected()) ? VARIANT_TRUE : VARIANT_FALSE;
    return(S_OK);
}

IFACEMETHODIMP
CSiUtilExUSB::SetTargetGo()
{
    const HRESULT hr = ::SetTargetGo();
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_SETTARGETGO, hr));
}

IFACEMETHODIMP
CSiUtilExUSB::SetTargetHalt()
{
    const HRESULT hr = ::SetTargetHalt();
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_SETTARGETHALT, hr));
}

IFACEMETHODIMP
CSiUtilExUSB::ISupportBanking(long* pnSupportedBanks)
{
    int nSupportedBanks = 0;
    const HRESULT hr = ::ISupportBanking(&nSupportedBanks);
    *pnSupportedBanks = nSupportedBanks;
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_ISUPPORTBANKING, hr));
}

IFACEMETHODIMP
CSiUtilExUSB::Download(BSTR sDownloadFile, VARIANT_BOOL blDeviceErase, VARIANT_BOOL blDownloadScratchPadSFLE, long nBankSelect, VARIANT_BOOL blLockFlash, VARIANT_BOOL blPersistFlash, long* usChecksum)
{
    const HRESULT hr = ::DownloadW(sDownloadFile, blDeviceErase, m_blDisableDialogBoxes, blDownloadScratchPadSFLE, nBankSelect, blLockFlash, blPersistFlash, usChecksum);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_DOWNLOAD, hr));
}

IFACEMETHODIMP
CSiUtilExUSB::TargetReset()
{
    const HRESULT hr = ::TargetReset();
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_TARGETRESET, hr));
}

IFACEMETHODIMP
CSiUtilExUSB::GetRAMMemory(long nStartAddress, long nLength, SAFEARRAY** ppMem)
{
    const HRESULT hr = ::GetMemory(::GetRAMMemory, nStartAddress, nLength, ppMem);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_GETRAMMEMORY, hr));
}

IFACEMETHODIMP
CSiUtilExUSB::GetXRAMMemory(long nStartAddress, long nLength, SAFEARRAY** ppMem)
{
    const HRESULT hr = ::GetMemory(::GetXRAMMemory, nStartAddress, nLength, ppMem);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_GETXRAMMEMORY, hr));
}

IFACEMETHODIMP
CSiUtilExUSB::GetCodeMemory(long nStartAddress, long nLength, SAFEARRAY** ppMem)
{
    const HRESULT hr = ::GetMemory(::GetCodeMemory, nStartAddress, nLength, ppMem);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_GETCODEMEMORY, hr));
}

IFACEMETHODIMP
CSiUtilExUSB::SetRAMMemory(long nStartAddress, SAFEARRAY** ppMem)
{
    const HRESULT hr = ::SetMemory(::SetRAMMemory, m_blDisableDialogBoxes, nStartAddress, ppMem);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_SETRAMMEMORY, hr));
}

IFACEMETHODIMP
CSiUtilExUSB::SetXRAMMemory(long nStartAddress, SAFEARRAY** ppMem)
{
    const HRESULT hr = ::SetMemory(::SetXRAMMemory, m_blDisableDialogBoxes, nStartAddress, ppMem);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_SETXRAMMEMORY, hr));
}

IFACEMETHODIMP
CSiUtilExUSB::SetCodeMemory(long nStartAddress, SAFEARRAY** ppMem)
{
#pragma warning(suppress: 4191)
    const HRESULT hr = ::SetMemory(reinterpret_cast<GETSETMEMORYFUNCTION>(::SetCodeMemory), m_blDisableDialogBoxes, nStartAddress, ppMem);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_SETCODEMEMORY, hr));
}

IFACEMETHODIMP
CSiUtilExUSB::FLASHErase(enum ECprotocol proto)
{
    const HRESULT hr = ::FLASHEraseUSB(m_pszSerialNumber, m_blDisableDialogBoxes, static_cast<int>(proto));
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_FLASHERASE, hr));
}

IFACEMETHODIMP
CSiUtilExUSB::GetFirmwareVersion(long* pnVersion)
{
    if(IsBadWritePtrT(pnVersion))
        return(RaiseCOMException(DISPID_SIUTILEX_GETFIRMWAREVERSION, E_POINTER));

    *pnVersion = ::GetUSBFirmwareVersion();
    return(S_OK);
}

IFACEMETHODIMP
CSiUtilExUSB::SetJTAGDeviceAndConnect(long nPowerTarget, long nDevicesBeforeTarget, long nDevicesAfterTarget, long IRBitsBeforeTarget, long IRBitsAfterTarget)
{
    if((static_cast<unsigned long>(nDevicesBeforeTarget) > MAXBYTE) || (static_cast<unsigned long>(nDevicesAfterTarget) > MAXBYTE) || (static_cast<unsigned long>(IRBitsBeforeTarget) > MAXWORD) || (static_cast<unsigned long>(IRBitsAfterTarget) > MAXWORD))
        return(RaiseCOMException(DISPID_SIUTILEX_SETJTAGDEVICEANDCONNECT, E_INVALIDARG));

    const HRESULT hr = ::SetJTAGDeviceAndConnectUSB(m_pszSerialNumber, static_cast<int>(nPowerTarget), m_blDisableDialogBoxes, static_cast<BYTE>(nDevicesBeforeTarget), static_cast<BYTE>(nDevicesAfterTarget), static_cast<WORD>(IRBitsBeforeTarget), static_cast<WORD>(IRBitsAfterTarget));
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_SETJTAGDEVICEANDCONNECT, hr));
}

IFACEMETHODIMP
CSiUtilExUSB::DebugDevices(long* dwDevices)
{
    if(IsBadWritePtrT(dwDevices))
        return(RaiseCOMException(DISPID_SIUTILEX_DEBUGDEVICES, E_INVALIDARG));

    DWORD dw;
    const HRESULT hr = ::USBDebugDevices(&dw);
    *dwDevices = static_cast<long>(dw);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_DEBUGDEVICES, hr));
}

IFACEMETHODIMP
CSiUtilExUSB::GetDeviceSN(long nDeviceNum, BSTR* psSerialNum)
{
    if(IsBadWritePtrT(psSerialNum))
        return(RaiseCOMException(DISPID_SIUTILEX_GETDEVICESN, E_INVALIDARG));

    *psSerialNum = nullptr;
    const char* str = nullptr;
    HRESULT hr = ::GetUSBDeviceSN(static_cast<DWORD>(nDeviceNum), &str);

    if(SUCCEEDED(hr))
        hr = BSTRFromStringA(str, psSerialNum);

    return((SUCCEEDED(hr)) ? S_OK : RaiseCOMException(DISPID_SIUTILEX_GETDEVICESN, hr));
}

IFACEMETHODIMP
CSiUtilExUSB::GetDeviceName(BSTR* psDeviceName)
{
    if(IsBadWritePtrT(psDeviceName))
        return(RaiseCOMException(DISPID_SIUTILEX_GETDEVICENAME, E_INVALIDARG));

    *psDeviceName = nullptr;
    const char* str;
    HRESULT hr = ::GetDeviceName(&str);

    if(SUCCEEDED(hr))
        hr = BSTRFromStringA(str, psDeviceName);

    return((SUCCEEDED(hr)) ? S_OK : RaiseCOMException(DISPID_SIUTILEX_GETDEVICENAME, hr));
}

IFACEMETHODIMP
CSiUtilExUSB::SetAppName(BSTR AppName)
{
    const HRESULT hr = ::SetAppNameW(AppName);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_SETAPPNAME, hr));
}

IFACEMETHODIMP
CSiUtilExUSB::GetScratchPadMemory(long nStartAddress, long nLength, SAFEARRAY** ppMem)
{
    const HRESULT hr = ::GetMemory(::GetScratchPadMemory, nStartAddress, nLength, ppMem);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_GETSCRATCHPADMEMORY, hr));
}

IFACEMETHODIMP
CSiUtilExUSB::SetScratchPadMemory(long nStartAddress, SAFEARRAY** ppMem)
{
#pragma warning(suppress: 4191)
    const HRESULT hr = ::SetMemory(reinterpret_cast<GETSETMEMORYFUNCTION>(::SetScratchPadMemory), m_blDisableDialogBoxes, nStartAddress, ppMem);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_SETSCRATCHPADMEMORY, hr));
}

IFACEMETHODIMP
CSiUtilExUSB::VerifyFlashDownload(BSTR sDownloadFile, long nBankSelect, VARIANT_BOOL* pblSuccess)
{
    const HRESULT hr = ::VerifyFlashDownloadW(sDownloadFile, m_blDisableDialogBoxes, nBankSelect, pblSuccess);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_VERIFYFLASHDOWNLOAD, hr));
}

IFACEMETHODIMP
CSiUtilExUSB::DownloadTxt(BSTR sDownloadFile, long nStartAddress, VARIANT_BOOL blDeviceErase, VARIANT_BOOL blLockFlash, VARIANT_BOOL blPersistFlash, long* usChecksum)
{
    const HRESULT hr = ::DownloadTxtW(sDownloadFile, nStartAddress, blDeviceErase, m_blDisableDialogBoxes, blLockFlash, blPersistFlash, usChecksum);
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_DOWNLOADTXT, hr));
}

IFACEMETHODIMP
CSiUtilExUSB::GetDeviceRevision(long* pPartRev)
{
    if(IsBadWritePtrT(pPartRev))
        return(RaiseCOMException(DISPID_SIUTILEX_GETDEVICEREVISION, E_POINTER));

    BYTE rev = 0;
    const HRESULT hr = ::GetDeviceRevision(&rev);
    *pPartRev = static_cast<long>(static_cast<unsigned long>(rev));
    return((SUCCEEDED(hr)) ? hr : RaiseCOMException(DISPID_SIUTILEX_GETDEVICEREVISION, hr));
}

IFACEMETHODIMP
CSiUtilExUSB::GetClassInfo(ITypeInfo** ppTI)
{
    ITypeLib*   pTlb = nullptr;
    HRESULT     hr;
    UINT        uIndex;

    if(IsBadWritePtrT(ppTI))
        return(E_POINTER);

    *ppTI = nullptr;

    if(SUCCEEDED(hr = m_pTypeInfo->GetContainingTypeLib(&pTlb, &uIndex)))
    {
        hr = pTlb->GetTypeInfoOfGuid(CLSID_SiUtilExUSB, ppTI);
        pTlb->Release();
    }

    return(hr);
}

IFACEMETHODIMP
CSiUtilExUSB::InterfaceSupportsErrorInfo(REFIID riid)
{
    __try
    {
        if(IsEqualIID(riid, IID_ISiUtilExUSB))
            return(S_OK);
    }
    __except((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        return(E_INVALIDARG);
    }

    return(S_FALSE);
}
