#include "precomp.h"

HRESULT GetMemory(GETSETMEMORYFUNCTION pfn, long nStartAddress, long nLength, SAFEARRAY** ppMem)
{
    SAFEARRAYBOUND  sab;
    HRESULT         hr;
    BYTE*           pByte = nullptr;

    if(IsBadWritePtrT(ppMem))
        return(E_POINTER);

    sab.cElements = static_cast<ULONG>(nLength);
    sab.lLbound   = 0;

    if(nullptr == (*ppMem = SafeArrayCreate(VT_UI1, 1, &sab)))
        return(E_OUTOFMEMORY);

    if(SUCCEEDED(hr = SafeArrayAccessData(*ppMem, reinterpret_cast<void**>(&pByte))))
    {
        hr = pfn(pByte, static_cast<DWORD>(nStartAddress), static_cast<unsigned int>(nLength));
        SafeArrayUnaccessData(*ppMem);
    }

    if(FAILED(hr))
    {
        SafeArrayDestroy(*ppMem);
        *ppMem = nullptr;
    }

    return(hr);
}

HRESULT SetMemory(GETSETMEMORYFUNCTION pfn, BOOL blDisableDialogBoxes, long nStartAddress, SAFEARRAY** ppMem)
{
    HRESULT hr;
    BYTE*   pByte   = nullptr;
    long    lLBound = 0, lUBound = -1;

    if(IsBadReadPtrT(ppMem))
        return(E_INVALIDARG);
    if((SafeArrayGetDim(*ppMem) != 1) || (SafeArrayGetElemsize(*ppMem) != 1))
        return(E_INVALIDARG);

    SafeArrayGetLBound(*ppMem, 1, &lLBound);
    SafeArrayGetUBound(*ppMem, 1, &lUBound);

    if(lLBound > lUBound)
        return(E_INVALIDARG);

    if(SUCCEEDED(hr = SafeArrayAccessData(*ppMem, reinterpret_cast<void**>(&pByte))))
    {
        #pragma warning(suppress: 4191)
        if(pfn == reinterpret_cast<GETSETMEMORYFUNCTION>(::SetCodeMemory))
        {
            hr = ::SetCodeMemory(pByte, static_cast<DWORD>(nStartAddress), static_cast<unsigned int>((lUBound - lLBound) + 1), blDisableDialogBoxes);
        }
        #pragma warning(suppress: 4191)
        else if(pfn == reinterpret_cast<GETSETMEMORYFUNCTION>(::SetScratchPadMemory))
        {
            hr = ::SetScratchPadMemory(pByte, static_cast<DWORD>(nStartAddress), static_cast<unsigned int>((lUBound - lLBound) + 1), blDisableDialogBoxes);
        }
        else
        {
            hr = pfn(pByte, static_cast<DWORD>(nStartAddress), static_cast<unsigned int>((lUBound - lLBound) + 1));
        }
        SafeArrayUnaccessData(*ppMem);
    }

    return(hr);
}

HRESULT DownloadW(PCWSTR sDownloadFile, VARIANT_BOOL blDeviceErase, BOOL blDisableDialogBoxes, VARIANT_BOOL nDownloadScratchPadSFLE, long nBankSelect, VARIANT_BOOL nLockFlash, VARIANT_BOOL blPersistFlash, long* usChecksum)
{
    HRESULT hr = E_OUTOFMEMORY;

    if(IsBadWritePtrT(usChecksum))
        return(E_POINTER);

    *usChecksum = 0;
    const int nLen = WideCharToMultiByte(CP_ACP, WC_NO_BEST_FIT_CHARS, sDownloadFile, -1, nullptr, 0, nullptr, nullptr);

    if(!nLen)
        return(HRESULT_FROM_WIN32(GetLastError()));

    __try
    {
        PSTR pFile = static_cast<PSTR>(_malloca(static_cast<size_t>(nLen) * sizeof(*pFile)));

        if(pFile)
        {
            if(WideCharToMultiByte(CP_ACP, WC_NO_BEST_FIT_CHARS, sDownloadFile, -1, pFile, nLen, nullptr, nullptr))
                hr = ::DownloadCS(pFile, (blDeviceErase != VARIANT_FALSE), blDisableDialogBoxes, (nDownloadScratchPadSFLE != VARIANT_FALSE), static_cast<int>(nBankSelect), (nLockFlash) ? 1 : 0, (blPersistFlash) ? TRUE : FALSE, reinterpret_cast<unsigned short*>(usChecksum));
            else
                hr = HRESULT_FROM_WIN32(GetLastError());

            _freea(pFile);
        }
    }
    __except((GetExceptionCode() == STATUS_STACK_OVERFLOW) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        _resetstkoflw();
    }

    return(hr);
}

HRESULT DownloadTxtW(PCWSTR sDownloadFile, long wStartAddress, VARIANT_BOOL blDeviceErase, BOOL blDisableDialogBoxes, VARIANT_BOOL blLockFlash, VARIANT_BOOL blPersistFlash, long* usChecksum)
{
    HRESULT hr = E_OUTOFMEMORY;

    if(IsBadWritePtrT(usChecksum))
        return(E_POINTER);

    *usChecksum = 0;
    const int nLen = WideCharToMultiByte(CP_ACP, WC_NO_BEST_FIT_CHARS, sDownloadFile, -1, nullptr, 0, nullptr, nullptr);

    if(!nLen)
        return(HRESULT_FROM_WIN32(GetLastError()));

    __try
    {
        PSTR pFile = static_cast<PSTR>(_malloca(static_cast<size_t>(nLen) * sizeof(*pFile)));

        if(pFile)
        {
            if(WideCharToMultiByte(CP_ACP, WC_NO_BEST_FIT_CHARS, sDownloadFile, -1, pFile, nLen, nullptr, nullptr))
                hr = ::DownloadTxt(pFile, static_cast<unsigned long>(wStartAddress), (blDeviceErase != VARIANT_FALSE), blDisableDialogBoxes, (blLockFlash) ? 1 : 0, (blPersistFlash) ? TRUE : FALSE, reinterpret_cast<unsigned short*>(usChecksum));
            else
                hr = HRESULT_FROM_WIN32(GetLastError());

            _freea(pFile);
        }
    }
    __except((GetExceptionCode() == STATUS_STACK_OVERFLOW) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        _resetstkoflw();
    }

    return(hr);
}

HRESULT SetAppNameW(LPCWSTR AppName)
{
    HRESULT hr = E_OUTOFMEMORY;
    const int nLen = WideCharToMultiByte(CP_ACP, WC_NO_BEST_FIT_CHARS, AppName, -1, nullptr, 0, nullptr, nullptr);

    if(!nLen)
        return(HRESULT_FROM_WIN32(GetLastError()));

    __try
    {
        PSTR pName = static_cast<PSTR>(_malloca(static_cast<size_t>(nLen) * sizeof(*pName)));

        if(pName)
        {
            if(WideCharToMultiByte(CP_ACP, WC_NO_BEST_FIT_CHARS, AppName, -1, pName, nLen, nullptr, nullptr))
                hr = ::SetAppName(pName);
            else
                hr = HRESULT_FROM_WIN32(GetLastError());

            _freea(pName);
        }
    }
    __except((GetExceptionCode() == STATUS_STACK_OVERFLOW) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        _resetstkoflw();
    }

    return(hr);
}

HRESULT VerifyFlashDownloadW(PCWSTR sDownloadFile, BOOL blDisableDialogBoxes, long nBankSelect, VARIANT_BOOL* vblSuccess)
{
    HRESULT hr = E_OUTOFMEMORY;

    if(IsBadWritePtrT(vblSuccess))
        return(E_POINTER);

    *vblSuccess = VARIANT_FALSE;
    const int nLen = WideCharToMultiByte(CP_ACP, WC_NO_BEST_FIT_CHARS, sDownloadFile, -1, nullptr, 0, nullptr, nullptr);

    if(!nLen)
        return(HRESULT_FROM_WIN32(GetLastError()));

    __try
    {
        PSTR pFile = static_cast<PSTR>(_alloca(static_cast<size_t>(nLen) * sizeof(*pFile)));

        if(pFile)
        {
            if(WideCharToMultiByte(CP_ACP, WC_NO_BEST_FIT_CHARS, sDownloadFile, -1, pFile, nLen, nullptr, nullptr))
            {
                if(SUCCEEDED(hr = ::VerifyFlashDownload(pFile, blDisableDialogBoxes, static_cast<int>(nBankSelect))))
                {
                    *vblSuccess = VARIANT_TRUE;
                }
                else
                {
                    if(hr == MAKE_HRESULT(SEVERITY_ERROR, 0x0100, 0x24))
                        hr = S_OK;
                }
            }
            else
            {
                hr = HRESULT_FROM_WIN32(GetLastError());
            }

            _freea(pFile);
        }
    }
    __except((GetExceptionCode() == STATUS_STACK_OVERFLOW) ? EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH)
    {
        _resetstkoflw();
    }

    return(hr);
}

HRESULT BSTRFromStringA(const char* pstr, BSTR* pbstr)
{
    const int nLen = MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED | MB_ERR_INVALID_CHARS, pstr, -1, nullptr, 0);

    if(!nLen)
    {
        *pbstr = nullptr;
        return(HRESULT_FROM_WIN32(GetLastError()));
    }

    if(nullptr != (*pbstr = SysAllocStringLen(nullptr, static_cast<UINT>(nLen) - 1U)))
    {
        if(MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED | MB_ERR_INVALID_CHARS, pstr, -1, *pbstr, nLen))
            return(S_OK);

        const HRESULT hr = HRESULT_FROM_WIN32(GetLastError());
        SysFreeString(*pbstr);
        *pbstr = nullptr;
        return(hr);
    }

    return(E_OUTOFMEMORY);
}
