#include "precomp.h"

#define FILENAME_ENTRY  MAKEINTRESOURCE(1)

static const LPCTSTR RegEntryTable[][3] =
{
    { TEXT("CLSID\\{19EB2E45-15EB-4D3B-A9DF-D2FE4D13ECD6}"),                    nullptr,                TEXT("SiUtilEx")                                },
    { TEXT("CLSID\\{19EB2E45-15EB-4D3B-A9DF-D2FE4D13ECD6}\\TypeLib"),           nullptr,                TEXT("{FC7C15A5-164C-4C5D-9AD3-92F224BB36B8}")  },
    { TEXT("CLSID\\{19EB2E45-15EB-4D3B-A9DF-D2FE4D13ECD6}\\ProgID"),            nullptr,                TEXT("SiUtilEx.SiUtilEx")                       },
    { TEXT("CLSID\\{19EB2E45-15EB-4D3B-A9DF-D2FE4D13ECD6}\\InprocServer32"),    nullptr,                FILENAME_ENTRY                                  },
    { TEXT("CLSID\\{19EB2E45-15EB-4D3B-A9DF-D2FE4D13ECD6}\\InprocServer32"),    TEXT("ThreadingModel"), TEXT("Both")                                    },
    { TEXT("SiUtilEx.SiUtilEx"),                                                nullptr,                TEXT("SiUtilEx.SiUtilEx")                       },
    { TEXT("SiUtilEx.SiUtilEx\\CLSID"),                                         nullptr,                TEXT("{19EB2E45-15EB-4D3B-A9DF-D2FE4D13ECD6}")  },
    { TEXT("CLSID\\{23C1FC6F-B4C0-4BB5-B66B-7ABA49A12923}"),                    nullptr,                TEXT("SiUtilExUSB")                             },
    { TEXT("CLSID\\{23C1FC6F-B4C0-4BB5-B66B-7ABA49A12923}\\TypeLib"),           nullptr,                TEXT("{FC7C15A5-164C-4C5D-9AD3-92F224BB36B8}")  },
    { TEXT("CLSID\\{23C1FC6F-B4C0-4BB5-B66B-7ABA49A12923}\\ProgID"),            nullptr,                TEXT("SiUtilEx.SiUtilExUSB")                    },
    { TEXT("CLSID\\{23C1FC6F-B4C0-4BB5-B66B-7ABA49A12923}\\InprocServer32"),    nullptr,                FILENAME_ENTRY                                  },
    { TEXT("CLSID\\{23C1FC6F-B4C0-4BB5-B66B-7ABA49A12923}\\InprocServer32"),    TEXT("ThreadingModel"), TEXT("Apartment")                               },
    { TEXT("SiUtilEx.SiUtilExUSB"),                                             nullptr,                TEXT("SiUtilEx.SiUtilExUSB")                    },
    { TEXT("SiUtilEx.SiUtilExUSB\\CLSID"),                                      nullptr,                TEXT("{23C1FC6F-B4C0-4BB5-B66B-7ABA49A12923}")  },
    { nullptr, nullptr, nullptr }
};

STDAPI DllUnregisterServer()
{
    intptr_t    idx;
    HRESULT     hResult = S_OK;
    HKEY        hKey;

    if(FAILED(UnRegisterTypeLib(LIBID_SiUtilExLib, 1, 0, LANG_NEUTRAL, SYS_WIN32)))
        hResult = SELFREG_E_TYPELIB;

    for(idx = 0; RegEntryTable[idx][0]; ++idx)
        ;

    while(--idx >= 0)
    {
        if(RegEntryTable[idx][1])
        {
            if(ERROR_SUCCESS != RegCreateKeyEx(HKEY_CLASSES_ROOT, RegEntryTable[idx][0], 0, nullptr, REG_OPTION_NON_VOLATILE, KEY_WRITE, nullptr, &hKey, nullptr))
            {
                hResult = S_FALSE;
                continue;
            }

            if(ERROR_SUCCESS != RegDeleteValue(hKey, RegEntryTable[idx][1]))
                hResult = S_FALSE;

            RegCloseKey(hKey);
        }
        else
        {
            if(ERROR_SUCCESS != RegDeleteKey(HKEY_CLASSES_ROOT, RegEntryTable[idx][0]))
                hResult = S_FALSE;
        }
    }

    return(hResult);
}

STDAPI DllRegisterServer()
{
    uintptr_t   idx = 0;
    ITypeLib*   ptlb;
    TCHAR       szFileName[MAX_PATH * 2];
    LONG        lError;
    BOOL        blIsTypeLibReg = FALSE;
    HKEY        hKey;

#ifndef UNICODE
    WCHAR  szFileNameW[MAX_PATH * 2];
#endif

    while(RegEntryTable[idx][0])
    {
        if(ERROR_SUCCESS == (lError = RegCreateKeyEx(HKEY_CLASSES_ROOT, RegEntryTable[idx][0], 0, nullptr, REG_OPTION_NON_VOLATILE, KEY_WRITE, nullptr, &hKey, nullptr)))
        {
            if(FILENAME_ENTRY == RegEntryTable[idx][2])
            {
                GetModuleFileName(g_hInst, szFileName, _countof(szFileName));

                if(!blIsTypeLibReg)
                {
#ifndef UNICODE
                    MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, szFileName, -1, szFileNameW, _countof(szFileName));
                    if(SUCCEEDED(LoadTypeLibEx(szFileNameW, REGKIND_REGISTER, &ptlb)))
#else
                    if(SUCCEEDED(LoadTypeLibEx(szFileName, REGKIND_REGISTER, &ptlb)))
#endif
                    {
                        blIsTypeLibReg = TRUE;
                        ptlb->Release();
                    }
                }

                lError = RegSetValueEx(hKey, RegEntryTable[idx][1], 0, REG_SZ, reinterpret_cast<const BYTE*>(szFileName), static_cast<DWORD>((_tcslen(szFileName) + 1) * sizeof(TCHAR)));
            }
            else if(RegEntryTable[idx][2])
            {
                lError = RegSetValueEx(hKey, RegEntryTable[idx][1], 0, REG_SZ, reinterpret_cast<const BYTE*>(RegEntryTable[idx][2]), static_cast<DWORD>((_tcslen(RegEntryTable[idx][2]) + 1) * sizeof(TCHAR)));
            }
            RegCloseKey(hKey);
        }

        if(lError != ERROR_SUCCESS)
        {
            DllUnregisterServer();
            return(SELFREG_E_CLASS);
        }

        ++idx;
    }

    return(S_OK);
}
